package cz.fidentis.analyst;

import cz.fidentis.analyst.face.HumanFace;
import cz.fidentis.analyst.kdtree.KdTree;
import cz.fidentis.analyst.mesh.core.MeshFacet;
import cz.fidentis.analyst.visitors.mesh.HausdorffDistance;
import cz.fidentis.analyst.visitors.mesh.HausdorffDistance.Strategy;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * A class for testing the efficiency of algorithms.
 * 
 * @author Radek Oslejsek
 */
public class EfficiencyTests {
    
    private static File girlFile = new File("src/test/resources/cz/fidentis/analyst/average_girl_17-20.obj");
    private static File boyFile = new File("src/test/resources/cz/fidentis/analyst/average_boy_17-20.obj");
    private static File faceFile2 = new File("src/test/resources/cz/fidentis/analyst/00002_01_ECA.obj");
    private static File faceFile4 = new File("src/test/resources/cz/fidentis/analyst/00004_01_ECA.obj");
    
    private static HumanFace face1;
    private static HumanFace face2;
    
    /**
     * Main method 
     * @param args Input arguments 
     * @throws IOException on IO error
     */
    public static void main(String[] args) throws IOException {
        face1 = new HumanFace(faceFile2);
        face2 = new HumanFace(faceFile4);
        
        System.out.println(measureKdTreeCreation(face1) + "\tmsec:\tKd-tree creation of first face");
        System.out.println(measureKdTreeCreation(face2) + "\tmsec:\tKd-tree creation of second face");
        
        boolean relativeDist = true;
        boolean printDetails = false;
        
        System.out.println();
        System.out.println("Hausdorff distance sequentially:");
        testAndPrint(face1, new HausdorffDistance(face2.getMeshModel(), Strategy.POINT_TO_POINT_ABSOLUTE, false, false, false), printDetails);
        testAndPrint(face1, new HausdorffDistance(face2.getMeshModel(), Strategy.POINT_TO_POINT, relativeDist, false, false), printDetails);
        testAndPrint(face1, new HausdorffDistance(face2.getMeshModel(), Strategy.POINT_TO_TRIANGLE_APPROXIMATE, relativeDist, false, false), printDetails);

        System.out.println();
        System.out.println("Hausdorff distance consurrently:");
        testAndPrint(face1, new HausdorffDistance(face2.getMeshModel(), Strategy.POINT_TO_POINT_ABSOLUTE, false, false, true), printDetails);
        testAndPrint(face1, new HausdorffDistance(face2.getMeshModel(), Strategy.POINT_TO_POINT, relativeDist, false, true), printDetails);
        testAndPrint(face1, new HausdorffDistance(face2.getMeshModel(), Strategy.POINT_TO_TRIANGLE_APPROXIMATE, relativeDist, false, true), printDetails);
    }
    
    protected static void testAndPrint(HumanFace face, HausdorffDistance vis, boolean printDetails) {
        long startTime = System.currentTimeMillis();
        face.getMeshModel().compute(vis);
        long endTime = System.currentTimeMillis();
        
        System.out.println(
                (endTime-startTime) + 
                "\tmsec: " + 
                //"\t" + (vis.inParallel() ? "concurrently" : "sequentially") +
                "\t" + (vis.relativeDistance()? "relative" : "absolute") +
                "\t" + vis.getStrategy()
        );
        
        if (printDetails) {
            Map<MeshFacet, List<Double>> results = vis.getDistances();
            for (MeshFacet facet: results.keySet()) {
                System.out.println(results.get(facet));
            }
        }
    }
    
    private static long measureKdTreeCreation(HumanFace face) {
        long startTime = System.currentTimeMillis();
        KdTree kdTree = new KdTree(face.getMeshModel().getFacets());
        return System.currentTimeMillis() - startTime;
    }
    
}
